//
// File: 12F629 DiSEqC.c
//
// Author: Ron Mount G7DOE
//
/*
 Description:
 
 22kHz DiSEqC controller for SUP2400 up-converter

 22kHz is a period of 45.4545usec, so there is a transition each half cycle ie at 22.727usec.
 The PIC12F629 allows a clock speed of 20MHz, giving an instruction time of 200nsec, but
 16MHz is to hand, giving an instruction time of 250nsec.
 We might like, then, an ON time of 22.5usec (90 cycles) and an OFF time of 23usec (92 cycles),
 which sets a frequency of 1/45.5usec = 21.98kHz which should be close enough as the standard allows 20%!

 In fact, there seems to be some variation in the time it takes to get into the interrupt handler,
 perhaps depending on whether the specific instruction in the main program is a one or two cycle instruction.
 
 Use a timer to drive an interrupt routine to set a background accessible variable, SIGstate, at this rate,
 then application code can just test the state of SIGstate to sort out its clock requirements.

 Using Timer0

 Requirements:

 Clock : 16MHz (Crystal)

 Target : 12F629

 Licence : Do as you would be done by. Free to all.

 Version:
 1.00 - Initial release, December 2015

*/

//
// Defines/Includes
// Built using the xc8 C compiler available free with MPLAB-X
//
#include "xc.h"
#pragma config BOREN = OFF, CPD = OFF, FOSC = HS, MCLRE = ON, WDTE = OFF, CP = OFF, PWRTE = OFF
//
// global variables
//
static unsigned char SIGstate=0, counter=203;
//
void init(void) {
   // Timer 0
   OPTION_REG =  0x00;
   nGPPU = 1;   //   : Pullups disabled (1).
   INTEDG = 0;  //   : Interrupt edge = low to high (0).
   T0CS = 0;    //   : Clock select Internal Fosc/4 (0).
   T0SE = 0;    //   : Timer 0 edge low to high (0).
   PSA = 1   ;  //Prescaler assigned to WDT (1) Timer0 (0).
   // PS  b2..0 = 110 : Prescale=1:128 (note PS=0-> 1:2 for TMR0).
   //       000 = 1:2
   //       001 = 1:4
   //       011 = 1:16
   // OPTION_REG |= 0x03; //no prescale
   TMR0 = counter;
}

//
void init_ports(void) {
     CMCON = 7; // no comparator active

     GPIO = 0;
     TRISIO = 0;  // 0=o/p.
}

//
void enable_interrupts(void) {
   // Timer 0 :Osc off, Internal clk, On,zero T0IF
   T0IE = 1;    // Timer 0 interrupt enabled
   GIE = 1;     // Global & Peripheral interrupt enable.
   
   PEIE = 0;
}

//
void disable_interrupts(void) {
   GIE = 0; // disable interrupts
}

// routine for binary ONE
void Mark(void) { // 11cyc @ 22kHz then 22cyc at zero
    while (SIGstate == 0) {};
    for (char i=1; i<12; i++) {
        // SIGstate = 1
        GPIO2 = 1; // set output
        while (SIGstate == 1) {};
        GPIO2 = 0;
        while (SIGstate == 0) {};
        }
    while (SIGstate == 0) {};
    for (char i=1; i<23; i++) {
        // SIGstate = 1
        while (SIGstate == 1) {};
        while (SIGstate == 0) {};
        } 
}
// routine for binary ZERO
void Space(void) { // 22 cyc @22kHz then 11cyc at zero
    // wait for 1
    while (SIGstate == 0) {};
    for (char i=1; i<23; i++) {
        // SIGstate = 1
        GPIO2 = 1; // set output
        while (SIGstate == 1) {};
        GPIO2 = 0;
        while (SIGstate == 0) {};
        }
    while (SIGstate == 0) {};
    for (char i=1; i<12; i++) {
        // SIGstate = 1
        while (SIGstate == 1) {};
        while (SIGstate == 0) {};
        }
}
// routine for byte (9 bits with parity)
void OutByte(unsigned char send){
    unsigned char sendbit=0, parity=0;
    // work out parity bit - sum of bits (including parity bit) is 1
    // squirt bits from <send> msb first then parity bit
    for (int i=7; i>=0; i--){
        sendbit = (send >> i) & 0x01;   // extract i'th bit
        parity = parity + sendbit;      // parity is sum of bits modulo 2 at this point
        if (parity>1) parity = 0;
        if (sendbit)
            Mark();
         else
            Space();
    }
    parity = 1 - parity;    // turns sum of bits into true parity bit
    if (parity)
        Mark();
    else
        Space();
}

void msec() {
    // delay one millisecond approximately
    // i.e. 22 cycles of 22kHz system clock
    while (SIGstate == 0) {};
    for (char i=1; i<23; i++) {
        // SIGstate = 1
        while (SIGstate == 1) {};
        while (SIGstate == 0) {};
        }
}

//
void Timer0_action(void) {
static unsigned long t = 0;
// do DiSEqC stuff in here (just the once)
// So, send xE20300 (ignore the reply), power on
// It is suggested on the web that this module uses non-standard sequences
// E20300 to turn on freq translation, E20200 to turn it off
// see http://www.slideshare.net/telesatelliteenglish/vitorsworkshop-22788090
// 
OutByte(0xE2);
OutByte(0x03);
OutByte(0x00);
}
//
// Start here
//
void main() {

   init_ports();
   init();
   enable_interrupts();
   // hang around for a while in case SUP has wandered off some distance
   for (int i=1; i<=100; i++) msec();   // 100msec
   // then send xE00002 for no reply 'standby' (might be xE00003 'power on', I suppose)
   Timer0_action();
   asm("sleep");
}

//
/*************************** ISR ROUTINE **************************************/
void interrupt ISR(void){
   // Timer 0: GIE is cleared on entry
   TMR0IF = 0;      // clear timer0 overflow bit.
   // Note count to overflow, loses 2cy when updating
   SIGstate = 1 - SIGstate;           // flip bit output
   counter = 174 + TMR0 + SIGstate;
   TMR0 = counter;  // From PIC Timer 0 calculator.
   // Note GIE set by RETFIE instruction (see assembler output).
}
   

